using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// CUAEditor example
//
// Base editor for a CUA (Custom Unity Asset)
namespace CUAEditorPlugin
{
    public class GundomController : MVRScript
    {
      
        public JSONStorableColor CubeColor;
        public JSONStorableBool Pose1;
        public JSONStorableBool Pose2;
        public JSONStorableBool Pose3;
        public JSONStorableBool Pose4;
        public JSONStorableBool Pose0;
        public JSONStorableBool Pose5;
        public JSONStorableBool Pose6;
        public JSONStorableBool Pose7;

        // My misc variables to find and edit the prefab made in Unity
        //擪̑傫
        private string objectNameSearchString = "G";
        public Transform ObjectRoot;
        private Transform cubeTR;
        private Renderer cubeRenderer;

        // Security and initialization
        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 3600;
        private Animator cubeAnime;
     

        private JSONStorableFloat rotateSpeed;
        private JSONStorableFloat cubePosition;

        // The usual init function for VAM
        public override void Init()
        {
            try
            {
                // We only authorize the script to be used by a CUA
                if (containingAtom.type != "CustomUnityAsset")
                {
                    logError("CUAEditor - Please add this script on a Custom Unity Asset ( script disabled ).");
                    return;
                }

                // Default color for both shapes
                HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);

                // Color selection for the cube
                CubeColor = new JSONStorableColor("Cube color", hsvc, CubeColorCallback);
                RegisterColor(CubeColor);
                CreateColorPicker(CubeColor);

                // Anim state for the Open Toggle
                Pose1 = new JSONStorableBool("Pose1", false, PlayOpenCallBack);
                Pose1.storeType = JSONStorableParam.StoreType.Full;
                CreateToggle(Pose1, false);
                RegisterBool(Pose1);

                // Anim state for the Close Toggle
                Pose2= new JSONStorableBool("Pose2", false, PlayCloseCallBack);
                Pose2.storeType = JSONStorableParam.StoreType.Full;
                CreateToggle(Pose2, false);
                RegisterBool(Pose2);


                // Anim state for the Move Toggle
                Pose3= new JSONStorableBool("Pose3", false, PlayMoveCallBack);
                Pose3.storeType = JSONStorableParam.StoreType.Full;
                CreateToggle(Pose3, false);
                RegisterBool(Pose3);

                // Anim state for the MoveUP Toggle
                Pose4= new JSONStorableBool("Pose4", false, PlayMoveupCallBack);
                Pose4.storeType = JSONStorableParam.StoreType.Full;
                CreateToggle(Pose4, false);
                RegisterBool(Pose4);

                // Anim state for the MoveDown Toggle
                Pose0= new JSONStorableBool("Pose0", false, PlayMovedownCallBack);
                Pose0.storeType = JSONStorableParam.StoreType.Full;
                CreateToggle(Pose0, false);
                RegisterBool(Pose0);


                // Anim state for the Turn Toggle
                Pose5 = new JSONStorableBool("Pose5", false, PlayTurnCallBack);
                Pose5.storeType = JSONStorableParam.StoreType.Full;
                CreateToggle(Pose5, false);
                RegisterBool(Pose5);

                // Anim state for the Turn Toggle
                Pose6 = new JSONStorableBool("Pose6", false, PlaySpinCallBack);
                Pose6.storeType = JSONStorableParam.StoreType.Full;
                CreateToggle(Pose6, false);
                RegisterBool(Pose6);

                // Anim state for the Turn Toggle
                Pose7 = new JSONStorableBool("Pose7", false, PlaySpineCallBack);
                Pose7.storeType = JSONStorableParam.StoreType.Full;
                CreateToggle(Pose7, false);
                RegisterBool(Pose7);

                //Xs[hXC_[
                rotateSpeed = new JSONStorableFloat("Play Speed", 0f, 0f, 5f);
                CreateSlider(rotateSpeed, false);
                RegisterFloat(rotateSpeed);

                cubePosition = new JSONStorableFloat("Cube Position", 0f, -1f, 1f);
                CreateSlider(cubePosition, true);
                RegisterFloat(cubePosition);

                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("Help",
                    "<color=#000><size=35><b>Help</b></size></color>\n\n" +
                    "<color=#333>" +
                    "<b>CUAEditor</b> is a tiny plugin to show you how you can create an editor for a CUA ( Custom Unity Asset ).\n\n" +
                    "Change the parameters on the left and you will see the CUA updating in realtime.\n\n" +
                    "<b>IMPORTANT :</b> If you switch objects in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
                    "</color>"
                );
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 1100.0f;
            }
            catch (Exception e)
            {
                SuperController.LogError("CUAEditor - Exception caught: " + e);
            }
        }

        // In this case we don't need that, but maybe you do ?
        void Start()
        {

        }

        // Used for the initialization of the editor.
        // When it is initialized, there's nothing done every frame
        void Update()
        {
            // If the editor is not initialized, we're gonna try to do it
            // I'm putting 3600 tries ( at 60fps it is 1min )
            if (initLoops < maxInitLoops && editorInitialized == false)
            {
                initLoops++;
                InitEditor();
            }

            // Triggering an error if we have reached our limit of tries
            if (initLoops == maxInitLoops)
            {
                logError("CUAEditor : This CUA could not be initialized properly after 3600 tries. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }

            //cubeAnime.SetBool("Open", true);
            //cubeAnime.SetBool("Close", false);
            cubeAnime.speed = rotateSpeed.val;
            cubeTR.transform.localPosition = new Vector3(0, cubePosition.val, 0);
            
            //cubeTR.transform.localPosition = new Vector3(0, 0, 0);
        }

        // The function initializing everything we need to update the prefab with the editor
        // You can imagine whatever you want here, the only limitations are the things available in VAM's build
        // In this example case, we need the main transform, a transform for the sphere and the cube
        //
        // Initially this function is made to return a bool state to see if it is properly initialized.
        // But since the CUA loader is asynchronous when you reload a scene and I don't have a way to identify when unity's prefab is loaded
        // I have to "brute force" the initialization several times until the script finds the transform I need
        // It implies that your prefab is done properly and has the good name ( this is why the maxInitLoops exists )
        private bool InitEditor()
        {
            // When the editor is initialized, we don't need to do that anymore
            if (editorInitialized == true) return true;

            // We need to find our main transform, getObjectRoot is gonna do that for us
            // the search is based on our variable objectNameSearchString
            // you need to create a prefab containing this name in Unity
            // This method works for unparented and parented objects
            // ( Thanks for the reParentObject tip meshed ! )
            ObjectRoot = getObjectRoot(containingAtom.reParentObject);
            if (ObjectRoot == null)
            {
                return false;
            }

            // Finding my and cube transforms
          
            cubeTR = ObjectRoot.Find("GundomS");
            cubeAnime = cubeTR.GetComponent<Animator>();
          
                  
            // I found my transforms, now the rest
            if (cubeTR)
            {
                // Finding my and cube renderers
                
                cubeRenderer = cubeTR.GetComponent<Renderer>();

                // I found my renderers components, let's finish this
                if (cubeRenderer)
                {
                    // Everything is good, I can consider my editor initialized
                    editorInitialized = true;

                    // I now need initialize my different objects based on the informations in my JSONStorables
                    // You could go crazy and make unified functions to control the whole object instead
                    CubeColorCallback(CubeColor);
                    PlayOpenCallBack(Pose1.val);
                    PlayCloseCallBack(Pose2.val);
                    PlayMoveCallBack(Pose3.val);
                    PlayMoveupCallBack(Pose4.val);
                    PlayMovedownCallBack(Pose0.val);
                    PlayTurnCallBack(Pose5.val);
                    PlaySpinCallBack(Pose6.val);
                    PlaySpineCallBack(Pose7.val);
                    logDebug("CUAEditor : editor initialized properly");
                    return true;
                }
                else
                {
                    logError("CUAEditor : your CUA has a transform with the good name but does not have a renderer.");
                    initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                    return false;
                }
            }
            else
            {
                logError("CUAEditor : your CUA is missing mandatory components ( transform name cube and sphere ).");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }

            return false;
        }

        // Recursive function searching for a transform containing the string set in objectNameSearchString
        // It is used the first time for the initialization of the script
        private Transform getObjectRoot(Transform parent)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(objectNameSearchString))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        
        // Callback triggered when you change the color for the cube (or you can call it manually)
        protected void CubeColorCallback(JSONStorableColor color)
        {
            if (cubeRenderer)
            {
                Color newColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, 1.0f);
                cubeRenderer.material.SetColor("_Color", newColor);
            }
        }

        // Callback triggered when you change the visibility for the sphere (or you can call it manually)
       // protected void PlayOpenCallBack(bool visibility)
             protected void PlayOpenCallBack(bool play)
        {
            if (cubeTR)
            {
                //cubeTR.gameObject.SetActive(visibility);
                cubeAnime.SetBool("Pose1", play);
            }
        }
        //protected void PlayCloseCallBack(bool visibility)
            protected void PlayCloseCallBack(bool play)
        {
            if (cubeTR)
            {
                //cubeTR.gameObject.SetActive(visibility);
                cubeAnime.SetBool("Pose2", play);
            }
        }

        //protected void PlayCloseCallBack(bool visibility)
            protected void PlayMoveCallBack(bool play)
        {
            if (cubeTR)
            {
                //cubeTR.gameObject.SetActive(visibility);
                cubeAnime.SetBool("Pose3", play);
            }
        }

        // protected void PlayOpenCallBack(bool visibility)
        protected void PlayMoveupCallBack(bool play)
        {
            if (cubeTR)
            {
                //cubeTR.gameObject.SetActive(visibility);
                cubeAnime.SetBool("Pose4", play);
            }
        }
        //protected void PlayCloseCallBack(bool visibility)
        protected void PlayMovedownCallBack(bool play)
        {
            if (cubeTR)
            {
                //cubeTR.gameObject.SetActive(visibility);
                cubeAnime.SetBool("Pose0", play);
            }
        }

        //protected void PlayCloseCallBack(bool visibility)
        protected void PlayTurnCallBack(bool play)
        {
            if (cubeTR)
            {
                //cubeTR.gameObject.SetActive(visibility);
                cubeAnime.SetBool("Pose5", play);
            }
        }

        //protected void PlayCloseCallBack(bool visibility)
        protected void PlaySpinCallBack(bool play)
        {
            if (cubeTR)
            {
                //cubeTR.gameObject.SetActive(visibility);
                cubeAnime.SetBool("Pose6", play);
            }
        }
        //protected void PlayCloseCallBack(bool visibility)
        protected void PlaySpineCallBack(bool play)
        {
            if (cubeTR)
            {
                //cubeTR.gameObject.SetActive(visibility);
                cubeAnime.SetBool("Pose7", play);
            }
        }
        // **************************
        // Local Tools... just because I'm lazy
        // **************************
        private void logDebug(string debugText)
        {
            SuperController.LogMessage(debugText);
        }

        private void logError(string debugText)
        {
            SuperController.LogError(debugText);
        }
    }
}
